import { Component, OnInit } from '@angular/core';
import { UntypedFormGroup, UntypedFormControl, Validators } from '@angular/forms';
import { lastValueFrom } from 'rxjs';
import { FiData } from './fi-data';
import { JhaUnsavedChangesService } from '@jha/jharesponsive/jha-unsaved-changes';


@Component({
	// tslint:disable-next-line:component-selector
	selector: 'crud',
	templateUrl: './crud.component.html',
	styleUrls: ['./crud.component.scss']
})
export class CrudComponent implements OnInit {
	public fiAvailable: boolean = false;
	public editing: boolean = false;
	public adding: boolean = false;
	public fiList: FiData[] = [];
	public selectedRoutingNumber: string = '';
	public selectedFi: FiData = new FiData('', '', '', '', '', '', '', '', '', null, '', null);
	public popupNotificationText: string = '';
	public showPopupNotification: boolean = false;
	public showDeleteQuestion: boolean = false;
	public editForm: UntypedFormGroup = new UntypedFormGroup({
		'institutionName': new UntypedFormControl('', Validators.required),
		'envVariable': new UntypedFormControl(''),
		'eesUrl': new UntypedFormControl(''),
		'productName': new UntypedFormControl(''),
		'billingId': new UntypedFormControl(''),
		'partnerId': new UntypedFormControl(''),
		'routingNumber': new UntypedFormControl('', [Validators.required, Validators.pattern('^((0[0-9])|(1[0-2])|(2[1-9])|(3[0-2])|(6[1-9])|(7[0-2])|80)([0-9]{7})$')]),
		'notes': new UntypedFormControl('')
	});

	get institutionName(): any { return this.editForm.get('institutionName'); }
	get envVariable(): any { return this.editForm.get('envVariable'); }
	get eesUrl(): any { return this.editForm.get('eesUrl'); }
	get productName(): any { return this.editForm.get('productName'); }
	get billingId(): any { return this.editForm.get('billingId'); }
	get partnerId(): any { return this.editForm.get('partnerId'); }
	get routingNumber(): any { return this.editForm.get('routingNumber'); }
	get notes(): any { return this.editForm.get('notes'); }

	constructor(private unsavedChangesService: JhaUnsavedChangesService) {}

	public ngOnInit(): void {

		// START OF DUMMY DATA
		this.fiList = [
			new FiData(
				'Community Bank of Dayton',
				'PROD',
				'https://lookup.attuswebsolutions.com/attusprod/WatchDOGNSA/searchservice.asmx',
				'iPay',
				'30050',
				'72115678910',
				'101000019',
				'',
				'',
				null,
				'',
				null
			),
			new FiData(
				'Mission Federal Credit Union',
				'PROD',
				'https://lookup.attuswebsolutions.com/attusprod/WatchDOGNSA/searchservice.asmx',
				'iPay',
				'12345',
				'89473627343',
				'114923756',
				'',
				'',
				null,
				'',
				null
			)
		];
		// END OF DUMMY DATA

		this.fiAvailable = this.fiList.length > 0;
		this.selectedRoutingNumber = this.fiList[0].RoutingNumber;
		this.updateSelectedAccount();
	}

	// canDeactivate for unsaved changes guard
	public async canDeactivate() {
		if (this.editForm.dirty) {
			const promptResult =  this.unsavedChangesService.promptUnsavedChangesDialog(this.editForm.valid);
			
			let promptUserResult = await lastValueFrom(promptResult);
			
			if (promptUserResult === 'Save') {
				// Save changes in database, then return true
				console.log('Saving changes');
				alert('The user chose to save changes, so you should capture the changes and save them to a database here.');
				return true;
			} else if (promptUserResult === 'DontSave') {
				console.log('Dont save changes');
				return true;
			} else if (promptUserResult === 'LoseChanges') {
				console.log('Losing changes');
				return true;
			} else {
				console.log('Returning to editing without leaving page');
				return false;
			};
		} else {
			return true;
		}
	}

	public updateSelectedAccount(): void {
		this.editing = false;
		const filteredList = this.fiList.filter((fi) => fi.RoutingNumber === this.selectedRoutingNumber);
		if (filteredList.length > 0) {
			this.selectedFi = filteredList[0];
		}
		this.setEditDataFromSelectedFi();
	}

	public startEditing(): void {
		this.setEditDataFromSelectedFi();
		this.editing = true;
	}

	public startAdding(): void {
		this.selectedRoutingNumber = '';
		this.setEditDataFromScratch();
		this.adding = true;
		this.editing = true;
	}

	public saveChanges(): void {
		this.editing = false;

		if (!this.adding) {
			this.saveEditData();

			// Display popup notification
			this.popupNotificationText = 'Edited ' + this.institutionName.value;
			this.showPopupNotification = true;
		} else {
			this.fiList.push(
				new FiData(
					this.institutionName.value,
					this.envVariable.value,
					this.eesUrl.value,
					this.productName.value,
					this.billingId.value,
					this.partnerId.value,
					this.routingNumber.value,
					this.notes.value,
					'', null, '', null)
			);
			this.adding = false;
			this.fiAvailable = true;

			// Display popup notification
			this.popupNotificationText = 'Added ' + this.institutionName.value;
			this.showPopupNotification = true;

			// Update the selected account number. We do this after a brief
			// delay to allow the master list time to finish adding the new
			// account before we select it.
			window.setTimeout(() => {
				this.selectedRoutingNumber = this.routingNumber.value;
				this.updateSelectedAccount();
			}, 100);
		}

		// Update database here
		this.editForm.markAsPristine();
		this.editForm.markAsUntouched();
	}

	public cancelChanges(): void {
		this.setEditDataFromSelectedFi();
		this.editing = false;
		if (this.adding) {
			this.adding = false;
			if (this.fiList.length > 0) {
				this.selectedRoutingNumber = this.fiList[0].RoutingNumber;
				this.updateSelectedAccount();
			}
		}
	}

	public deleteQuestionClosed(dialogResult: string): void {
		if (dialogResult === 'Delete') {
			this.deleteFi();
		}
	}

	private deleteFi(): void {
		const index: number = this.fiList.findIndex((fi) => fi.RoutingNumber === this.selectedRoutingNumber);
		if (index >= 0) {
			const fiName = this.fiList[index].InstitutionName;
			this.fiList.splice(index, 1);
			this.selectedRoutingNumber = '';
			if (this.fiList.length > 0) {
				this.selectedRoutingNumber = this.fiList[0].RoutingNumber;
				this.updateSelectedAccount();

			} else {
				this.fiAvailable = false;
			}
			this.popupNotificationText = 'Deleted ' + fiName;
			this.showPopupNotification = true;

			// Update database here
		}
	}

	private setEditDataFromScratch(): void {
		this.editForm.reset();
		this.institutionName.setValue('');
		this.envVariable.setValue('');
		this.eesUrl.setValue('');
		this.productName.setValue('');
		this.billingId.setValue('');
		this.partnerId.setValue('');
		this.routingNumber.setValue('');
		this.notes.setValue('');
	}

	private setEditDataFromSelectedFi(): void {
		this.editForm.reset();
		this.institutionName.setValue(this.selectedFi.InstitutionName);
		this.envVariable.setValue(this.selectedFi.EnvVariable);
		this.eesUrl.setValue(this.selectedFi.EesUrl);
		this.productName.setValue(this.selectedFi.ProductName);
		this.billingId.setValue(this.selectedFi.BillingId);
		this.partnerId.setValue(this.selectedFi.PartnerId);
		this.routingNumber.setValue(this.selectedFi.RoutingNumber);
		this.notes.setValue(this.selectedFi.Notes);
	}

	private saveEditData(): void {
		this.selectedFi.InstitutionName = this.institutionName.value;
		this.selectedFi.EnvVariable = this.envVariable.value;
		this.selectedFi.EesUrl = this.eesUrl.value;
		this.selectedFi.ProductName = this.productName.value;
		this.selectedFi.BillingId = this.billingId.value;
		this.selectedFi.PartnerId = this.partnerId.value;
		this.selectedFi.RoutingNumber = this.routingNumber.value;
		this.selectedFi.Notes = this.notes.value;
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
