import { Component, OnInit } from '@angular/core';
import { JhaButtonGroupOption } from '@jha/jharesponsive/jha-buttons';
import { JhaThemingService } from '@jha/jharesponsive/jha-theme';
import { DataService } from '../../../services/data/data.service';
import { TranHistoryDetails } from '../../../classes/tran-history-details';
import { JhaPubSubService, JhaPubSubMessage } from '@jha/jharesponsive/jha-responsive-core';

import Chart from 'chart.js/auto';

@Component({
	selector: 'dashboard',
	templateUrl: './dashboard.component.html',
	styleUrls: ['./dashboard.component.scss'],
	providers: [JhaThemingService]
})
export class DashboardComponent implements OnInit {
	public growthRange: JhaButtonGroupOption[] = [
		new JhaButtonGroupOption('Month', 'Month'),
		new JhaButtonGroupOption('Quarter', 'Quarter'),
		new JhaButtonGroupOption('Year', 'Year')
	];
	public selectedGrowthRange: string = 'Month';
	private subscription: any = null;
	public ticklerList: TranHistoryDetails[];
	public sortedTicklerList: TranHistoryDetails[];
	public sortColumn: string;
	public loading: boolean = false;
	public sortDirection: number = 1;
	public sortDescription: string = '';
    private growthChart: any = null;
    private animateCharts: Boolean = true;
    private chartFontColor: String = '';
    private chartGridColor: String = '';
    private chartBackgroundColor1: String = '';
    private chartBackgroundColor2: String = '';

	constructor(
		private themingService: JhaThemingService,
		private pubSubService: JhaPubSubService,
		private dataService: DataService
	) { }

	ngOnInit(): void {
		// Check if the user requests no animation
        const mediaQuery = window.matchMedia("(prefers-reduced-motion: reduce)");
        if (!mediaQuery || mediaQuery.matches) {
            this.animateCharts = false;
        }

		this.subscription = this.pubSubService.Stream.subscribe(pubSubMessage => this.processMessage(pubSubMessage));

		this.InitializeChart();

		this.loading = true;
		this.dataService.GetTransactions().then(results => {
			this.loading = false;
			this.ticklerList = results;
			this.sortedTicklerList = this.ticklerList.slice();
			this.sortHistory('TranDate');
		});
	}

	public processMessage(pubSubMessage: JhaPubSubMessage): void {
		if (pubSubMessage.message === 'THEME_CHANGED') {
			this.InitializeChart();
		}
	}
	
	// Initialize Chart
	InitializeChart(): void {
		setTimeout(() => {
			this.setUpGrowthChart();
		}, 100);
	}

	// Build or update chart
    private setUpGrowthChart() {
        const canvas = document.querySelector('#growthChart');
		this.chartFontColor = this.themingService.GetChartFontColor();
		this.chartGridColor = this.themingService.GetChartGridColor();
		this.chartBackgroundColor1 = this.themingService.GetChartDataSeriesColor(1);
		this.chartBackgroundColor2 = this.themingService.GetChartDataSeriesColor(2);
		
        if (canvas) {
            if (this.growthChart) {
                // If chart already exists, update it with current theme's chart colors
                this.growthChart.data.datasets[0].backgroundColor = this.chartBackgroundColor1;
                this.growthChart.data.datasets[1].backgroundColor = this.chartBackgroundColor2;
                this.growthChart.options.plugins.legend.labels.color = this.chartFontColor;
                this.growthChart.options.scales.x.ticks.color = this.chartFontColor;
                this.growthChart.options.scales.x.grid.color = this.chartGridColor;
                this.growthChart.options.scales.y.ticks.color = this.chartFontColor;
                this.growthChart.options.scales.y.grid.color = this.chartGridColor;
                this.growthChart.update();
            } else {
                // Otherwise create a new chart using current theme's chart colors
                this.growthChart = new Chart(canvas as any, {
                    type: 'bar',
                    data: {
                        labels: ['January', 'February', 'March', 'April', 'May', 'June', 'July'],
                        datasets: [
                            {
                                label: 'Primary',
                                backgroundColor: this.chartBackgroundColor1 as any,
                                data: [65, 59, 80, 81, 56, 55, 40]
                            },
                            {
                                label: 'Secondary',
                                backgroundColor: this.chartBackgroundColor2,
                                data: [28, 48, 40, 19, 86, 27, 90]
                            }
                        ]
                    },
                    options: {
                        ...(!this.animateCharts && { animation: false }),
                        responsive: true,
                        maintainAspectRatio: false,
                        plugins: {
                            legend: {
                                labels: {
                                    color: this.chartFontColor as any
                                }
                            }
                        },
                        scales:
                        {
                            x: {
                                grid: { color: this.chartGridColor as any },
                                ticks: { color: this.chartFontColor as any }
                            },
                            y: {
                                grid: { color: this.chartGridColor as any },
                                ticks: { color: this.chartFontColor as any }
                            }
                        }
                    }
                });
            }
        }
    }

	// Sort the tickler history
	public sortHistory(sortColumn: string): any {
		// Reverse the sort direction
		this.sortDirection *= -1;
		const sortDirection: number = this.sortDirection;

		// Remember the sort column
		this.sortColumn = sortColumn;

		this.sortedTicklerList.sort((itemA: TranHistoryDetails, itemB: TranHistoryDetails) => {
			// Compare ascendening
			let sortResult = 0;
			if (itemA[sortColumn] < itemB[sortColumn]) {
				sortResult = -1;
			} else if (itemA[sortColumn] > itemB[sortColumn]) {
				sortResult = 1;
			}

			// Adjust for sort direction
			sortResult *= sortDirection;

			return sortResult;
		});

		this.updateSortDescription();
	}

	private updateSortDescription(): void {
		this.sortDescription = 'Sorted by ';
		switch (this.sortColumn) {
			case 'TranDate': {
				this.sortDescription += 'Date';
				break;
			}
			case 'Payee': {
				this.sortDescription += 'Payee';
				break;
			}
		}
		this.sortDescription += (this.sortDirection === -1 ? ' (decreasing) ' : ' (increasing) ');
	}
}
