import { Component, OnInit, NgZone, OnDestroy } from '@angular/core';
import { JhaPubSubService, JhaPubSubMessage } from '@jha/jharesponsive/jha-responsive-core';
import { JhaButtonGroupOption } from '@jha/jharesponsive/jha-buttons';
import { DataService } from '../../../services/data/data.service';
import { Employee } from '../../../classes/employee';
import { takeWhile } from 'rxjs/operators';

@Component({
	selector: 'large-datasets',
	templateUrl: './large-datasets.component.html',
	styleUrls: ['./large-datasets.component.scss']
})
export class LargeDatasetsComponent implements OnInit, OnDestroy {
	public zone: NgZone;
	public approachOptions: JhaButtonGroupOption[] = [
		new JhaButtonGroupOption('Pagination', 'Pagination'),
		new JhaButtonGroupOption('Infinite Scroll', 'Infinite Scroll')
	];
	public approach: string = 'Pagination';
	public loading: boolean = false;
	public EmployeePageIndex: number = 1;
	public EmployeeIndex: number = 0;
	public EmployeesPerPage: number = 8;
	public EmployeeDataset: Employee[] = null;
	public EmployeeTotalCount: number = 0;
	public PaginationInfo: string = 'loading';
	private alive: boolean = true;

	private subscription: any = null;

	constructor(
		private dataService: DataService,
		private pubSubService: JhaPubSubService,
	) {
		this.zone = new NgZone({ enableLongStackTrace: false });
	}

	ngOnInit(): void {
		// Subscribe to pub/sub
		this.subscription = this.pubSubService.Stream.pipe(takeWhile(() => this.alive)).subscribe(pubSubMessage => this.processMessage(pubSubMessage));

		this.dataService.GetEmployeeTotalCount().then(
			data => {
				this.EmployeeTotalCount = data;

				this.EmployeePageIndex = 1;
				this.updatePaginationEmployeeList();
			});

	}

	ngOnDestroy(): void {
		// On component destroy, stop tracking scroll
		this.pubSubService.Stream.next(new JhaPubSubMessage('TRACKSCROLL', false));
		this.alive = false;
		this.subscription.unsubscribe();
	}

	public setApproach(): void {
		if (this.approach === 'Pagination') {
			this.SetUpPagination();
		} else {
			this.SetUpInfiniteScroll();
		}
	}

	public pageChanged(event: any): void {
		this.EmployeePageIndex = event.page;
		this.updatePaginationEmployeeList();
	}

	public SetUpPagination(): void {
		this.EmployeePageIndex = 1;
		this.updatePaginationEmployeeList();

		// Publish that we want to stop tracking scrolling
		this.pubSubService.Stream.next(new JhaPubSubMessage('TRACKSCROLL', false));
	}

	public SetUpInfiniteScroll(): void {
		this.EmployeePageIndex = 1;
		this.updatePaginationEmployeeList();

		// Publish that we're wanting to track scrolling
		this.pubSubService.Stream.next(new JhaPubSubMessage('TRACKSCROLL', true));
	}

	// If user has scrolled to the bottom, add more employees
	processMessage(pubSubMessage: JhaPubSubMessage): void {
		if (pubSubMessage.message === 'SCROLLED') {
			this.addMoreEmployees();
		}
	}

	private updatePaginationEmployeeList(): void {
		this.loading = true;
		let startIndex = (this.EmployeePageIndex - 1) * this.EmployeesPerPage;
		if (startIndex < 0) {
			startIndex = 0;
		}
		let endIndex = startIndex + this.EmployeesPerPage - 1;
		if (endIndex > this.EmployeeTotalCount - 1) {
			endIndex = this.EmployeeTotalCount - 1;
		}
		this.PaginationInfo = 'Employees ' + (startIndex + 1).toString() + ' - ' + (endIndex + 1).toString() +
			' of ' + this.EmployeeTotalCount.toString();

		this.dataService.GetEmployees(startIndex, this.EmployeesPerPage).then(
			data => {
				this.EmployeeDataset = data;
				this.loading = false;
			});
	}

	private addMoreEmployees(): void {
		if (this.approach === 'Infinite Scroll' && !this.loading) {
			const startIndex = this.EmployeeDataset.length;
			let count = this.EmployeesPerPage;
			if (startIndex + count > this.EmployeeTotalCount) {
				count = this.EmployeeTotalCount - startIndex;
			}
			if (count > 0) {
				this.loading = true;
				this.dataService.GetEmployees(startIndex, count).then(
					data => {
						this.zone.run(() => {
							this.EmployeeDataset = this.EmployeeDataset.concat(data);
							this.PaginationInfo = 'Employees 1 - ' + this.EmployeeDataset.length.toString() + ' of ' + this.EmployeeTotalCount.toString();
							this.loading = false;
						});
					}
				);
			}
		}
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
