import { AfterViewInit, Component, OnDestroy, OnInit } from '@angular/core';
import { UntypedFormGroup, UntypedFormControl, Validators } from '@angular/forms';
import { Subscription } from 'rxjs';

import { DataService } from '../../../../../services/data/data.service';
import { JhaDynamicComponentLoaderService } from '@jha/jharesponsive/jha-responsive-core';
import { SelectItem } from 'primeng/api';
import { AddressData } from '../../classes/addressData';

@Component({
	selector: 'edit-address',
	templateUrl: './edit-address.component.html',
	styleUrls: ['./edit-address.component.scss']
})
export class EditAddressComponent implements OnInit, AfterViewInit, OnDestroy {
	public stateList: SelectItem[] = [];
	public addressData: AddressData = new AddressData(1, 1, []);
	private callback: any;
	public formIsDirty: boolean = false;
	private stackedViewDataSubscription: Subscription;
	private stackedViewActionSubscription: Subscription;

	public editForm: UntypedFormGroup  = new UntypedFormGroup({
		'shippingName': new UntypedFormControl('', Validators.required),
		'shippingStreet': new UntypedFormControl('', Validators.required),
		'shippingCity': new UntypedFormControl('', Validators.required),
		'shippingState': new UntypedFormControl('', Validators.required),
		'shippingZip': new UntypedFormControl('', [Validators.required, Validators.pattern('^[0-9]{5}(?:[0-9]{4})?$')])
	});
	get shippingName(): any { return this.editForm.get('shippingName'); }
	get shippingStreet(): any { return this.editForm.get('shippingStreet'); }
	get shippingCity(): any { return this.editForm.get('shippingCity'); }
	get shippingState(): any { return this.editForm.get('shippingState'); }
	get shippingZip(): any { return this.editForm.get('shippingZip'); }

	constructor(
		private dataService: DataService,
		private dynamicComponentLoaderService: JhaDynamicComponentLoaderService
	) {	}

	ngOnInit(): void {
		// Get list of states
		this.dataService.GetStateList().then((stateList) => {
			this.stateList = [ { 'label': 'Select a state', 'value': '' } ];
			stateList.forEach((stateOption) => { this.stateList.push({ 'label': stateOption.name, 'value': stateOption.value }); });
		});

		// Subscribe to stacked view data service
		this.stackedViewDataSubscription = this.dynamicComponentLoaderService.stackedViewData.subscribe(data => {
			if (data) {
				this.addressData = new AddressData(data.SelectedAddressId, data.EditingAddressId, data.AddressList);

				const filteredAddressList = data.AddressList.filter((address) => address.Id === data.EditingAddressId);
				if (filteredAddressList.length > 0) {
					this.editForm.controls['shippingName'].setValue(filteredAddressList[0].Name);
					this.editForm.controls['shippingStreet'].setValue(filteredAddressList[0].Street);
					this.editForm.controls['shippingCity'].setValue(filteredAddressList[0].City);
					this.editForm.controls['shippingState'].setValue(filteredAddressList[0].State);
					this.editForm.controls['shippingZip'].setValue(filteredAddressList[0].ZipCode);
				}
			}
		});

		// Subscribe to stacked view action service
		this.stackedViewActionSubscription = this.dynamicComponentLoaderService.stackedViewAction.subscribe(action => {
			if (action) {
				this.processMessage(action);
			}
		});
	}

	ngAfterViewInit(): void {
		// Subscribe to stacked view data service
		this.dynamicComponentLoaderService.stackedViewData.subscribe(data => {
			if (data) {
				this.addressData = new AddressData(data.SelectedAddressId, data.EditingAddressId, data.AddressList);

				const filteredAddressList = data.AddressList.filter((address) => address.Id === data.EditingAddressId);
				if (filteredAddressList.length > 0) {
					this.editForm.controls['shippingName'].setValue(filteredAddressList[0].Name);
					this.editForm.controls['shippingStreet'].setValue(filteredAddressList[0].Street);
					this.editForm.controls['shippingCity'].setValue(filteredAddressList[0].City);
					this.editForm.controls['shippingState'].setValue(filteredAddressList[0].State);
					this.editForm.controls['shippingZip'].setValue(filteredAddressList[0].ZipCode);
				}
			}
		});
	}

	// Unsubscribe from subscription on destroy
	ngOnDestroy(): void {
		this.stackedViewDataSubscription.unsubscribe();
		this.stackedViewActionSubscription.unsubscribe();
	}

	// Listen for events when unsaved changes dialog needs to save and close views
	private processMessage(stackedViewAction: any): void {
		if (stackedViewAction.stackedViewPayload.stackedViewName === 'EditAddressComponent') {
			if (stackedViewAction.stackedViewPayload.stackedViewAction === 'JHA_STACKEDVIEWS_UNSAVEDCHANGES') {
				if (this.editForm.dirty) {
					this.formIsDirty = true;
				} else {
					this.formIsDirty = false;
				}
				stackedViewAction.stackedViewPayload.stackedViewCallback(this.formIsDirty);
			} else if (stackedViewAction.stackedViewPayload.stackedViewAction === 'JHA_STACKEDVIEWS_SAVEANDCLOSEVIEW') {
				this.saveAddress();
				stackedViewAction.stackedViewPayload.stackedViewCallback(true);
			}
		}
	}

	// Save changes to address
	public saveAddress(): void {
		const filteredAddressList = this.addressData.AddressList.filter((address) => address.Id === this.addressData.EditingAddressId);
		if (filteredAddressList.length > 0) {
			filteredAddressList[0].Name = this.shippingName.value;
			filteredAddressList[0].Street = this.shippingStreet.value;
			filteredAddressList[0].City = this.shippingCity.value;
			filteredAddressList[0].State = this.shippingState.value;
			filteredAddressList[0].ZipCode = this.shippingZip.value;
		}
		this.removeStackedView(this.addressData);
	}

	// Cancel address changes
	public cancelChanges(): void {
		this.removeStackedView();
	}

	// Close this view
	public closeEditAddress(): void {
		this.removeStackedView();
	}

	// Remove this view in the stack
	private removeStackedView(data?: any): void {
		this.dynamicComponentLoaderService.removeStackedView('EditAddressComponent', data);
	}

	// Close all stacked views
	public removeAllStackedViews(): void {
		this.dynamicComponentLoaderService.removeAllStackedViews();
	}

}
