import { AfterContentInit, Component, OnDestroy, OnInit } from '@angular/core';
import { UntypedFormGroup, UntypedFormControl, Validators } from '@angular/forms';
import { SelectItem } from 'primeng/api';
import { Subscription, lastValueFrom } from 'rxjs';

// Services
import { DataService } from '../../../services/data/data.service';
import { JhaDynamicComponentLoaderService } from '@jha/jharesponsive/jha-responsive-core';
import { JhaUnsavedChangesService } from '@jha/jharesponsive/jha-unsaved-changes';

// Classes
import { Address } from './classes/address';
import { AddressData } from './classes/addressData';

// Components used in stacked layers
import { SelectAddressComponent } from './components/select-address/select-address.component';

@Component({
	selector: 'stacked-views',
	templateUrl: './stacked-views.component.html',
	styleUrls: ['./stacked-views.component.scss']
})
export class StackedViewsComponent implements OnInit, AfterContentInit, OnDestroy {
	public viewsStacked: boolean = false;
	public selectedSectionId: string = 'ShippingAddress';
	public showQuestionWithDestructiveOption: boolean = false;
	public SelectAddressComponentClass = SelectAddressComponent;
	public stateList: SelectItem[] = [];
	public selectedAddressId: number = null;
	public addressList: Address[] = [];
	public addressData: AddressData = new AddressData(1, 1, []);
    	private stackedViewSubscription: Subscription;

	public editForm: UntypedFormGroup = new UntypedFormGroup({
		'shippingName': new UntypedFormControl('', Validators.required),
		'shippingStreet': new UntypedFormControl('', Validators.required),
		'shippingCity': new UntypedFormControl('', Validators.required),
		'shippingState': new UntypedFormControl('', Validators.required),
		'shippingZip': new UntypedFormControl('', [Validators.required, Validators.pattern('^[0-9]{5}(?:[0-9]{4})?$')])
	});
	get shippingName(): any { return this.editForm.get('shippingName'); }
	get shippingStreet(): any { return this.editForm.get('shippingStreet'); }
	get shippingCity(): any { return this.editForm.get('shippingCity'); }
	get shippingState(): any { return this.editForm.get('shippingState'); }
	get shippingZip(): any { return this.editForm.get('shippingZip'); }

	constructor(
		private dataService: DataService,
		private dynamicComponentLoaderService: JhaDynamicComponentLoaderService,
		private unsavedChangesService: JhaUnsavedChangesService
	) {	}

	ngOnInit(): void {
        this.resetForm();

		// Get list of states
		this.dataService.GetStateList().then((stateList) => {
			this.stateList = [ { 'label': 'Select a state', 'value': '' } ];
			stateList.forEach((stateOption) => { this.stateList.push({ 'label': stateOption.name, 'value': stateOption.value }); });
		});

		// Build list of addresses
		this.addressList.push(new Address(1, 'Christopher Erickson', '142 Elm St', 'Morristown', 'NJ', '07960'));
		this.addressList.push(new Address(2, 'Olive Diaz', '9923 Park Ln', 'Murraysville', 'PA', '15668'));
		this.addressList.push(new Address(3, 'George Jones', '43323 FM 455', 'Mansfield', 'IA', '44812'));

		// Build out data object to share across views
		this.addressData.SelectedAddressId = null;
		this.addressData.AddressList = this.addressList;
	}

	ngAfterContentInit(): void {
		// Subscribe to stacked view data service
		this.stackedViewSubscription = this.dynamicComponentLoaderService.stackedViewData.subscribe(data => {
			if (data) {
				this.addressData = new AddressData(data.SelectedAddressId, data.EditingAddressId, data.AddressList);

				const filteredAddressList = data.AddressList.filter((address) => address.Id === data.SelectedAddressId);
				if (filteredAddressList.length > 0) {
					this.editForm.controls['shippingName'].setValue(filteredAddressList[0].Name);
					this.editForm.controls['shippingStreet'].setValue(filteredAddressList[0].Street);
					this.editForm.controls['shippingCity'].setValue(filteredAddressList[0].City);
					this.editForm.controls['shippingState'].setValue(filteredAddressList[0].State);
					this.editForm.controls['shippingZip'].setValue(filteredAddressList[0].ZipCode);
				}

				this.editForm.markAsDirty();
				this.editForm.markAsTouched();
			}
		});
	}

	ngOnDestroy(): void {
		this.removeAllStackedViews();
		this.stackedViewSubscription.unsubscribe();
	}

	// canDeactivate for unsaved changes guard
	public async canDeactivate(): Promise<boolean> {
		// Test if stacked views are opened
		return this.dynamicComponentLoaderService.stackedViewLevel().then((stackedLevel) => {
			if (stackedLevel > 0) {
				// If stacked views exists, check if any have unsaved changes
				return this.dynamicComponentLoaderService.checkForUnsavedChanges().then((unsavedChanges) => {
					if (!unsavedChanges && !this.editForm.dirty) {
						return true;
					} else {
						return this.promptUnsavedChangesDialog(stackedLevel);
					}
				});
			// Otherwise, test if base page's form is dirty
			} else {
				// If base page form is dirty, prompt user for action
				if (this.editForm.dirty) {
					return this.promptUnsavedChangesDialog(stackedLevel);
				// Otherwise return true and continue with route changes
				} else {
					return true;
				}
			}
		});
	}

	public async promptUnsavedChangesDialog(stackLevel: number): Promise<boolean> {
		if (stackLevel > 0) {
			this.viewsStacked = true;
		} else {
			this.viewsStacked = false;
		}

		const promptResult =  this.unsavedChangesService.promptUnsavedChangesDialog();

		let promptUserResult = await lastValueFrom(promptResult);
	
		if (promptUserResult === 'Save') {
			//  If the views are stacked, run service method to save and close all stacked views
			if (this.viewsStacked) {
				const saveAndClose =  this.dynamicComponentLoaderService.saveAndCloseStackedViews();

				let saveAndCloseResult = await lastValueFrom(saveAndClose);

				if (saveAndCloseResult) {
					console.log('Saving changes...');
					alert('The user chose to save changes, so you should capture the changes and save them to a database here.');
					// Stacked views changes saved
					// Save all changes in database, then return true
					return true;
				}
			// Otherwise just save changes to base page and continue with route change
			} else {
				// Save changes to base page in database, then return true
				console.log('Saving changes...');
				alert('The user chose to save changes, so you should capture the changes and save them to a database here.');
				return true;
			}
		} else if (promptUserResult === 'DontSave') {
			console.log('Losing changes');
			return true;
		} else {
			console.log('Not leaving page');
			return false;
		}
	}
	
	// Reset form
	private resetForm() {
		this.editForm.markAsPristine();
		this.editForm.markAsUntouched();
	}

	// Cancel changes
	public cancelChanges(): void {
		alert('This cancels the edit session, so your app would typically navigate to a different view here');
	}

	// Save changes
	public saveChanges(): void {
		if (this.editForm.valid) {
			alert('All fields were successfully validated, so your app would typically navigate to a different view here');
        }
	}

	// Add new stacked view
	public addStackedView(stackedComponent: any, stackedComponentName: string): void {
		if (stackedComponent) {
			this.dynamicComponentLoaderService.addStackedView(stackedComponent, stackedComponentName, this.addressData);
		}
	}

	// Close all stacked views
	public removeAllStackedViews(): void {
		this.dynamicComponentLoaderService.removeAllStackedViews();
	}
}
