import { Component, OnInit, ViewEncapsulation } from '@angular/core';
import { UntypedFormGroup, UntypedFormControl, Validators } from '@angular/forms';
import { MenuItem, SelectItem } from 'primeng/api';
import { lastValueFrom } from 'rxjs';
import { DataService } from '../../../services/data/data.service';
import { JhaUnsavedChangesService } from '@jha/jharesponsive/jha-unsaved-changes';
import { ShippingMethod } from '../../../classes/shipping-method';
import { JhaButtonGroupOption } from '@jha/jharesponsive/jha-buttons';

@Component({
	// tslint:disable-next-line:component-selector
	selector: 'steps',
	templateUrl: './steps.component.html',
	styleUrls: ['./steps.component.scss'],
	encapsulation: ViewEncapsulation.None
})
export class StepsComponent implements OnInit {
	public stepItems: MenuItem[];
	public stepIndex: number = 0;
	public stateList: SelectItem[] = [];
	public expMonths: SelectItem[] = [];
	public expYears: SelectItem[] = [];
	public shippingMethodList: ShippingMethod[] = [];
	public shippingMethodOptions: JhaButtonGroupOption[] = [];
	public editForm: UntypedFormGroup = new UntypedFormGroup({
		'shippingName': new UntypedFormControl('Nancy Davolio', Validators.required),
		'shippingStreet': new UntypedFormControl('123 Main St', Validators.required),
		'shippingCity': new UntypedFormControl('San Diego', Validators.required),
		'shippingState': new UntypedFormControl('CA', Validators.required),
		'shippingZip': new UntypedFormControl('92110', [Validators.required, Validators.pattern('^[0-9]{5}(?:[0-9]{4})?$')]),
		'billingName': new UntypedFormControl('Nancy Davolio', Validators.required),
		'billingStreet': new UntypedFormControl('123 Main St', Validators.required),
		'billingCity': new UntypedFormControl('San Diego', Validators.required),
		'billingState': new UntypedFormControl('CA', Validators.required),
		'billingZip': new UntypedFormControl('92110', [Validators.required, Validators.pattern('^[0-9]{5}(?:[0-9]{4})?$')]),
		'paymentCardNumber': new UntypedFormControl('9876543298209382', Validators.required),
		'paymentCardCvc': new UntypedFormControl('0293', Validators.required),
		'paymentCardExpMonth': new UntypedFormControl('', Validators.required),
		'paymentCardExpYear': new UntypedFormControl('', Validators.required),
		'shippingMethod': new UntypedFormControl(null, Validators.required),
	});
	get shippingName(): any { return this.editForm.get('shippingName'); }
	get shippingStreet(): any { return this.editForm.get('shippingStreet'); }
	get shippingCity(): any { return this.editForm.get('shippingCity'); }
	get shippingState(): any { return this.editForm.get('shippingState'); }
	get shippingZip(): any { return this.editForm.get('shippingZip'); }
	get billingName(): any { return this.editForm.get('billingName'); }
	get billingStreet(): any { return this.editForm.get('billingStreet'); }
	get billingCity(): any { return this.editForm.get('billingCity'); }
	get billingState(): any { return this.editForm.get('billingState'); }
	get billingZip(): any { return this.editForm.get('billingZip'); }
	get paymentCardNumber(): any { return this.editForm.get('paymentCardNumber'); }
	get paymentCardCvc(): any { return this.editForm.get('paymentCardCvc'); }
	get paymentCardExpMonth(): any { return this.editForm.get('paymentCardExpMonth'); }
	get paymentCardExpYear(): any { return this.editForm.get('paymentCardExpYear'); }
	get shippingMethod(): any { return this.editForm.get('shippingMethod'); }

	constructor(
		private dataService: DataService,
		private unsavedChangesService: JhaUnsavedChangesService
	) {}

	ngOnInit(): void {
		// Get list of states
		this.dataService.GetStateList().then((stateList) => {
			this.stateList = [];
			stateList.forEach((stateOption) => { this.stateList.push({ 'label': stateOption.name, 'value': stateOption.value }); });
		});

		// Create list of shipping methods
		this.shippingMethodList = new Array<ShippingMethod>();
		this.shippingMethodList.push(new ShippingMethod('USPS', 'US Postal Service', '4.95'));
		this.shippingMethodList.push(new ShippingMethod('UPS Ground', 'UPS Ground', '6.95'));
		this.shippingMethodList.push(new ShippingMethod('UPS Overnight', 'UPS Overnight', '18.90'));

		// Add these to the list of shipping options
		for (let i = 0; i < this.shippingMethodList.length; i++ ) {
			this.shippingMethodOptions.push(new JhaButtonGroupOption(this.shippingMethodList[i].name, this.shippingMethodList[i]));
		}

		// Auto-select the first item in the list
		this.shippingMethod.setValue(this.shippingMethodList[0]);

		// Set up list of expiration years, along with default exp month/year
		const thisDate = new Date();
		const thisMonth = thisDate.getMonth() + 1;
		const thisYear = thisDate.getFullYear();
		for (let x = 1; x <= 12; x++) {
			let monthStr = x.toString();
			while (monthStr.length < 2) {
				monthStr = '0' + monthStr;
			}
			this.expMonths.push({ 'label': monthStr, 'value': x });
		}
		for (let x = 0; x <= 5; x++) {
			this.expYears.push({ 'label': (thisYear + x).toString(), 'value': thisYear + x });
		}
		this.paymentCardExpMonth.setValue(thisMonth);
		this.paymentCardExpYear.setValue(thisYear + 1);

		// Define step items
		this.stepItems = [{
			label: 'Shipping Address',
			command: (event: any) => {
				this.stepIndex = 0;
			}
		},
		{
			label: 'Billing Address',
			command: (event: any) => {
				this.stepIndex = 1;
			}
		},
		{
			label: 'Payment Method',
			command: (event: any) => {
				this.stepIndex = 2;
			}
		},
		{
			label: 'Shipping',
			command: (event: any) => {
				this.stepIndex = 3;
			}
		},
		{
			label: 'Confirm Order',
			command: (event: any) => {
				this.stepIndex = 4;
			}
		}];
	}

	// canDeactivate for unsaved changes guard
	public async canDeactivate() {
		if (this.editForm.dirty) {
			const promptResult =  this.unsavedChangesService.promptUnsavedChangesDialog(this.editForm.valid);
			
			let promptUserResult = await lastValueFrom(promptResult);
			
			if (promptUserResult === 'Save') {
				// Save changes in database, then return true
				console.log('Saving changes');
				alert('The user chose to save changes, so you should capture the changes and save them to a database here.');
				return true;
			} else if (promptUserResult === 'DontSave') {
				console.log('Dont save changes');
				return true;
			} else if (promptUserResult === 'LoseChanges') {
				console.log('Losing changes');
				return true;
			} else {
				console.log('Returning to editing without leaving page');
				return false;
			};
		} else {
			return true;
		}
	}

	public back(): void {
		if (this.stepIndex > 0) {
			this.stepIndex--;
		}
	}

	public next(): void {
		if (this.stepIndex < 4) {
			this.stepIndex++;
		}
	}

	public placeOrder(): void {
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}
}
