import { Component, OnDestroy, OnInit, ViewChild } from '@angular/core';
import { NgForm } from '@angular/forms';
import { lastValueFrom } from 'rxjs';

import { LogInOutService } from '../../../services/log-in-out/log-in-out.service';
import { DataService } from '../../../services/data/data.service';
import { JhaUnsavedChangesService } from '@jha/jharesponsive/jha-unsaved-changes';

import { User } from '../../../classes/user';
import { Account } from '../../../classes/account';

@Component({
	// tslint:disable-next-line:component-selector
	selector: 'wizard',
	templateUrl: './wizard.component.html',
	styleUrls: ['./wizard.component.scss']
})
export class WizardComponent implements OnInit, OnDestroy {
	public loading: boolean = false;
	public loadError: string = '';
	public users: User[];
	public selectedUser: User;
	public selectedUserId: string = '';
	public accounts: Account[];
	public selectedAccount: Account;
	public selectedAccountId: string = '';
	public selectedAccountAvailable: number = 0;
	public sendAmount: number = 0;
	public sendComment: string = '';
	public wizardTitle: string = 'Who are you sending money to?';
	public wizardStepIndex: number = 1;
	public wizardStepCount: number = 4;
	public wizardMaxStepVisited: number = 1;
	public wizardNextButtonText: string = 'Next >';
	public wizardNextButtonDisabled: boolean = true;
	public formHasUnsavedChanges: boolean = false;
	@ViewChild('form', {static: true}) ngForm: NgForm;
	public formChangeSubscription: any = null;

	constructor(
		private logInOutService: LogInOutService,
		private dataService: DataService,
		private unsavedChangesService: JhaUnsavedChangesService
	) { }

	ngOnInit(): void {
		this.GetUserList();
		this.GetAccountListForUser();
		this.WizardSetup();

		this.formChangeSubscription = this.ngForm.form.valueChanges.subscribe(change => {
			this.formHasUnsavedChanges = true;
		});
	}

	// canDeactivate for unsaved changes guard
	public async canDeactivate() {
		if (this.formHasUnsavedChanges) {
			const promptResult =  this.unsavedChangesService.promptUnsavedChangesDialog();
			
			let promptUserResult = await lastValueFrom(promptResult);
			
			if (promptUserResult === 'Save') {
				// Save changes in database, then return true
				console.log('Saving changes');
				alert('The user chose to save changes, so you should capture the changes and save them to a database here.');
				return true;
			} else if (promptUserResult === 'DontSave') {
				console.log('Dont save changes');
				return true;
			} else if (promptUserResult === 'LoseChanges') {
				console.log('Losing changes');
				return true;
			} else {
				console.log('Returning to editing without leaving page');
				return false;
			};
		} else {
			return true;
		}
	}

	ngOnDestroy(): void {
		this.formChangeSubscription.unsubscribe();
	}

	private GetUserList(): void {
		this.loading = true;
		this.dataService.GetAllUsers().then(
			data => {
				this.users = data;
				this.RemoveLoggedInUser();
				this.loading = false;
			});
	}

	private GetAccountListForUser(): void {
		this.loading = true;
		this.dataService.GetAccountsForUser().then(
			data => {
				this.accounts = data;
				this.loading = false;
			});
	}

	private RemoveLoggedInUser(): void {
		// Remove user that matches the logged-in user
		// since you can't send money to yourself
		const loggedInUser = this.logInOutService.GetLoggedInUser();
		if (loggedInUser) {
			let loggedInUserIndex = -1;
			for (let i = 0; i < this.users.length; i++) {
				if (this.users[i].id === loggedInUser.id) {
					loggedInUserIndex = i;
					break;
				}
			}
			if (loggedInUserIndex >= 0) {
				this.users.splice(loggedInUserIndex, 1);
			}
		}
	}

	public updateSelectedUser(): void {
		const filteredList = this.users.filter((user) => user.id === this.selectedUserId);
		if (filteredList.length > 0) {
			this.selectedUser = filteredList[0];
			this.wizardNextButtonDisabled = false;
		}
	}

	public updateSelectedAccount(): void {
		const filteredList = this.accounts.filter((account) => account.id === this.selectedAccountId);
		if (filteredList.length > 0) {
			this.selectedAccount = filteredList[0];
			this.selectedAccountAvailable = this.selectedAccount.amountInCents / 100;
			this.wizardNextButtonDisabled = false;
		}
	}

	public ChangeAmount(): void {
		this.wizardNextButtonDisabled =
			this.sendAmount < 1 || this.sendAmount > this.selectedAccountAvailable;
	}

	// Go to previous step of wizard
	public WizardBack(): void {
		if (this.wizardStepIndex > 1) {
			this.wizardStepIndex--;
		}
		this.WizardSetup();
	}

	// Go to next step of wizard
	public WizardNext(): void {
		this.wizardStepIndex++;
		this.WizardSetup();
	}

	// Go back to a specific step of the Add User wizard
	public wizardStep(stepNumber: number): void {
		if (stepNumber <= this.wizardMaxStepVisited) {
			if (stepNumber >= 1 && stepNumber <= this.wizardStepCount) {
				this.wizardStepIndex = stepNumber;
			}
			this.WizardSetup();
		}
	}

	// Set up to send more money
	public SendAgain(): void {
		this.wizardStepIndex = 1;
		this.selectedUser = null;
		this.selectedUserId = '';
		this.selectedAccount = null;
		this.selectedAccountId = '';
		this.selectedAccountAvailable = 0;
		this.sendAmount = 0;
		this.sendComment = '';
		this.GetUserList();
		this.GetAccountListForUser();
		this.WizardSetup();
	}

	// Setup for wizard
	private WizardSetup(): void {
		if (this.wizardStepIndex > this.wizardMaxStepVisited) {
			this.wizardMaxStepVisited = this.wizardStepIndex;
		}

		if (this.wizardStepIndex === 1) {
			this.wizardTitle = 'Who are you sending money to?';
			this.wizardNextButtonText = 'Next >';
			this.wizardNextButtonDisabled = this.selectedUserId === '';
		} else if (this.wizardStepIndex === 2) {
			this.wizardTitle = 'Send money from which account?';
			this.wizardNextButtonText = 'Next >';
			this.wizardNextButtonDisabled = this.selectedAccountId === '';
		} else if (this.wizardStepIndex === 3) {
			this.wizardTitle = 'What amount do you want to send?';
			this.wizardNextButtonText = 'Next >';
			this.wizardNextButtonDisabled =
				this.sendAmount < 1 || this.sendAmount > this.selectedAccountAvailable;

			window.setTimeout(() => {
				const sendAmtField = document.getElementById('SendAmount') as HTMLInputElement;
				sendAmtField.focus();
				sendAmtField.select();
			}, 100);
		} else if (this.wizardStepIndex === 4) {
			this.wizardTitle = 'Recap';
			this.wizardNextButtonText = 'Finish';
		} else if (this.wizardStepIndex === 5) {
			this.loading = true;
			this.dataService.SendMoney(this.selectedUser.userName, this.selectedAccountId, this.sendAmount, this.sendComment).then(
				data => {
					this.loading = false;
					this.wizardTitle = 'Money sent';
					this.formHasUnsavedChanges = false;
				});
		}
	}

	public openPattern(url: string): void {
		const win = window.open(url, '_blank');
		win.focus();
	}

}
